@@ -24,40 +24,45 @@ import pandas as pd
 
 from .env_utils import get_env_bool
 
-# Map SQL types to Python datatypes
+# Map SQL types to pandas nullable datatypes
 DATATYPE_MAP = {
-    "double": np.float64,
-    "float": float,
-    "int": int,
-    "longlong": int,
-    "smallint": int,
-    "datetime": str,
-    "var_string": str,
-    "varchar": str,
-    "char": str,
-    "blob": object,  # Not sure about this one..
-    "mediumblob": object,  # Not sure about this one..
-    "longblob": object,  # Not sure about this one..
-    "bigint": np.int64,
-    "shorttext": str,
-    "mediumtext": str,
-    "longtext": str,
-    "enum": str,
-    "tinyint": int,
-    "text": str,
-    "bool": bool,
-    "decimal": float,
-    "timestamp": str,
-    "bit": bool,
-    "mediumint": int,
-    "integer": int,
-    "int8": np.int64,
-    "int4": int,
-    "int3": int,
-    "int2": int,
-    "int1": int,
-    "middleint": int,
-    "serial": int,
+    "double": "Float64",
+    "float": "Float64",
+    "int": "Int64",
+    "longlong": "Int64",
+    "long": "Int64",
+    "smallint": "Int64",
+    "datetime": "string",
+    "var_string": "string",
+    "string": "string",
+    "varchar": "string",
+    "char": "string",
+    "blob": "object",  # Keep as object for binary data
+    "mediumblob": "object",  # Keep as object for binary data
+    "longblob": "object",  # Keep as object for binary data
+    "bigint": "Int64",
+    "shorttext": "string",
+    "mediumtext": "string",
+    "longtext": "string",
+    "enum": "string",
+    "tinyint": "Int64",
+    "tinyint(1)": "boolean",  # TODO: actually, this could be anything from -127 to 128 but we'll just assume boolean for now
+    "tiny": "Int64",
+    "text": "string",
+    "bool": "boolean",
+    "decimal": "Float64",
+    "newdecimal": "Float64",
+    "timestamp": "string",
+    "bit": "boolean",
+    "mediumint": "Int64",
+    "integer": "Int64",
+    "int8": "Int64",
+    "int4": "Int64",
+    "int3": "Int64",
+    "int2": "Int64",
+    "int1": "Int64",
+    "middleint": "Int64",
+    "serial": "Int64",
 }
 
 # TODO: HANDLE ALL THESE:
@@ -67,6 +72,69 @@ DATATYPE_MAP = {
 # 'TIME', 'TIME2', 'TIMESTAMP', 'TIMESTAMP2', 'TINY', 'TINY_BLOB', 'VARCHAR',
 # 'VAR_STRING', 'YEAR'
 
+
+def refine_dtype(data_type: str, column_type: str):
+    """
+    You can get this information from e.g.
+
+        SELECT DATA_TYPE, COLUMN_TYPE
+        FROM INFORMATION_SCHEMA.COLUMNS
+        WHERE TABLE_SCHEMA = 'your_database'
+        AND TABLE_NAME = 'your_table';
+
+    """
+    data_type = data_type.lower()
+    column_type = column_type.lower()
+
+    # 1) boolean synonyms / tinyint(1)
+    if data_type in ("bool", "boolean"):
+        return "boolean"
+    if data_type == "tinyint":
+        # Treat tinyint(1) as boolean ONLY if truly boolean
+        if column_type == "tinyint(1)":
+            # optional: whitelist by name convention or check constraints
+            return "boolean"
+        # otherwise keep as small int
+        return "Int8"
+
+    # 2) BIT
+    if data_type == "bit":
+        if column_type == "bit(1)":
+            return "boolean"
+        # For bit(n>1): either keep bytes or convert yourself later
+        return "object"
+
+    # 3) unsigned integers
+    if data_type in ("tinyint", "smallint", "mediumint", "int", "integer", "bigint"):
+        unsigned = "unsigned" in column_type
+        if unsigned:
+            return {
+                "tinyint": "UInt8",
+                "smallint": "UInt16",
+                "mediumint": "UInt32",
+                "int": "UInt32",
+                "integer": "UInt32",
+                "bigint": "UInt64",
+            }[data_type]
+        else:
+            return {
+                "tinyint": "Int8",
+                "smallint": "Int16",
+                "mediumint": "Int32",
+                "int": "Int32",
+                "integer": "Int32",
+                "bigint": "Int64",
+            }[data_type]
+
+    # 4) decimals: consider precision
+    if data_type in ("decimal", "newdecimal", "numeric"):
+        # If you care about exactness, pick 'object' (Decimal) instead of Float64
+        return "Float64"
+
+    # otherwise use base
+    return "object"
+
+
 # Get a lookup of what all the data_types are
 FIELD_TYPE = mariadb.constants.FIELD_TYPE
 FIELD_TYPE_DICT = {
@@ -134,14 +202,18 @@ class DataFrameFast(pd.DataFrame):
             for dt_col in dt_cols.keys():
                 # I think this is okay if we were careful to have the column's data
                 # in the current session's datetime format
-                new_col = (
-                    df0.loc[:, dt_col]
-                    # Convert to the correct time zone if necessary
-                    .dt.tz_convert(session_timezone)
-                    # Convert to a string for mariadb to digest
-                    .dt.strftime("%Y-%m-%d %H:%M:%S.%f")
-                    .astype("string")
-                )
+                # Convert timezone-aware timestamps to MariaDB-compatible strings
+                col_data = df0.loc[:, dt_col]
+
+                # Convert to the target timezone if necessary
+                if col_data.dt.tz is not None:
+                    # Convert to target timezone, then remove timezone info for MariaDB
+                    col_data = col_data.dt.tz_convert(session_timezone).dt.tz_localize(
+                        None
+                    )
+
+                # Convert to string format that MariaDB accepts
+                new_col = col_data.dt.strftime("%Y-%m-%d %H:%M:%S.%f").astype("string")
 
                 # Directly assign the converted datetime strings without intermediate assignments
                 # This avoids the pandas FutureWarning about incompatible dtype assignment
@@ -150,6 +222,16 @@ class DataFrameFast(pd.DataFrame):
                 SeriesCtor = cast(type[pd.Series], pd.Series)
                 df0[dt_col] = SeriesCtor(new_col, dtype="string", index=df0.index)
 
+        # Convert boolean columns to integers for MariaDB TINYINT(1) compatibility
+        bool_cols: dict[str, Any] = {
+            col: dtype
+            for col, dtype in df0.dtypes.to_dict().items()
+            if str(dtype) == "boolean" or isinstance(dtype, bool)
+        }
+        for col in bool_cols:
+            # Convert boolean to int (True->1, False->0, pd.NA->None)
+            df0[col] = df0[col].astype("Int64")
+
         # Convert np.int64 columns to np.int32 since MySQL cannot accept np.int64 as a param
         small_int64cols = {
             col: dtype
@@ -193,16 +275,19 @@ class DataFrameFast(pd.DataFrame):
             # Identify boolean columns and convert them to int (1 for True, 0 for False)
             # to avoid the warning: ('Warning', 1366, "Incorrect integer value:
             # 'False' for column michael_db.cats.is_success at row 1")
-            bool_cols = df0.select_dtypes(include=["bool"]).columns
-            df0[bool_cols] = df0[bool_cols].astype(int)
+            bool_cols_to_exclude: list[str] = list(
+                df0.select_dtypes(include=["bool", "boolean"]).columns
+            )
+            df0[bool_cols_to_exclude] = df0[bool_cols_to_exclude].astype(int)
 
             # Convert to csv for uploading to the server as a file, which is faster
             # for some reason
             # the fillna is to handle NULLs
-            df0.fillna("\\N").to_csv(
+            df0.to_csv(
                 tmp_filepath,
                 index=False,
                 header=True,
+                na_rep="\\N",  # ← this prints pd.NA/NaN as \N
                 quotechar=ENCLOSURE_CHAR,
                 quoting=csv.QUOTE_ALL,
                 sep=FIELD_DELIMITER,
@@ -366,23 +451,45 @@ def read_sql_table(
     assert query is not None
 
     # Use the main tonydbc to retrieve this with
-    records, type_codes = con.get_data(
-        query=query, no_tracking=True, return_type_codes=True
-    )
+    records = con.get_data(query=query, no_tracking=True)
+    type_codes = con.get_type_codes(query=query)
 
-    # TODO: convert all cols
+    # Convert SQL types to pandas nullable datatypes
+    dtype_conversions = {}
+    for col, sql_type in type_codes.items():
+        sql_type_lower: str = sql_type.lower()
 
-    # KLUDGE: for now, just convert any BIT fields to bool
-    bit_cols = {k: bool for k, v in type_codes.items() if v.upper() == "BIT"}
+        # Special handling for TINY type - need to check if it's boolean
+        # In MariaDB, BOOL is an alias for TINYINT(1), so TINY fields might be boolean
+        if sql_type_lower == "tiny":
+            # For now, assume TINY is boolean if it's a single-byte field
+            # This is a heuristic - ideally we'd check the actual column definition
+            dtype_conversions[col] = "boolean"
+        elif sql_type_lower in DATATYPE_MAP:
+            dtype_conversions[col] = DATATYPE_MAP[sql_type_lower]
+        else:
+            # Raise error for unknown types
+            raise ValueError(
+                f"Unknown SQL type '{sql_type_lower}' for column '{col}'. Please add it to DATATYPE_MAP."
+            )
+
+    # Special handling for BIT fields (convert binary to boolean)
+    bit_cols = {k: v for k, v in type_codes.items() if v.upper() == "BIT"}
 
     if len(records) > 0:
         df = DataFrameFast.from_records(records)
 
+        # Convert BIT fields from binary to boolean
         for k in bit_cols.keys():
             # Convert b'\x01' to True and b'\x00' to False
             df[k] = df[k].apply(
                 lambda cell: bool(int.from_bytes(cell, byteorder="big"))
             )
+
+        # Apply nullable datatype conversions
+        for col, target_dtype in dtype_conversions.items():
+            if col in df.columns:
+                df[col] = df[col].astype(target_dtype)
     elif "CALL" not in query.upper():
         # We also have to return the columns names in case records is []
         """Use a trick to get the column names,
@@ -399,7 +506,10 @@ def read_sql_table(
             columns = [v[0] for v in cursor.description]
 
         df = DataFrameFast(columns=columns)
-        df = df.astype(bit_cols)
+        # Apply both bit column conversions and nullable datatype conversions for empty DataFrames
+        all_conversions = {**bit_cols, **dtype_conversions}
+        if all_conversions:
+            df = df.astype(all_conversions)
     else:
         # We cannot get the columns if it was a stored procedure
         return DataFrameFast(columns=[])
