@@ -53,7 +53,7 @@ import time
 import zoneinfo
 from contextlib import contextmanager
 from types import TracebackType
-from typing import Any, Callable, Literal, overload
+from typing import Any, Callable
 
 import dateutil
 import filelock
@@ -136,6 +136,7 @@ class _TonyDBCOnlineOnly:
     ipath: str
     media_to_deserialize: dict[str, list[str]]
     _audit_db: "_TonyDBCOnlineOnly | None"
+    __production_databases: list[str]
 
     def __init__(
         self,
@@ -561,7 +562,7 @@ class _TonyDBCOnlineOnly:
         # Make new connections
         self.__enter__()
 
-    def iso_timestamp_to_session_time_zone(self, iso_timestamp_string: str):
+    def iso_timestamp_to_session_time_zone(self, iso_timestamp_string: str) -> str:
         """e.g. converts an ISO-formatted string to the database session's time zone,
         self.default_tz, which is useful when adding to a mariadb TIMESTAMP
         which assumes the time is formatted as the session time zone.
@@ -584,7 +585,9 @@ class _TonyDBCOnlineOnly:
             iso_ts = ts.astimezone(self.default_tz)  # dateutil.tz.UTC
             return iso_ts.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3]
 
-    def write_dataframe(self, df, table_name, if_exists="replace", index=False):
+    def write_dataframe(
+        self, df: pd.DataFrame, table_name: str, if_exists: str = "replace", index=False
+    ) -> None:
         """Write all rows of a dataframe to the database
         Assumes the fields are the same in number and data type
         """
@@ -597,7 +600,7 @@ class _TonyDBCOnlineOnly:
             index=index,
         )
 
-    def update_table(self, table_name: str, df):
+    def update_table(self, table_name: str, df: pd.DataFrame):
         """Perform an UPDATE with appropriate serialization.
 
         This is the spiritual sister of `append_to_table` (i.e. it uses the same fancy semantics
@@ -630,7 +633,12 @@ class _TonyDBCOnlineOnly:
         else:
             columns_to_serialize = []
 
-        df_serialized = serialize_table(df, col_dtypes, columns_to_serialize)
+        df_serialized = serialize_table(
+            cur_df=df,
+            col_dtypes=col_dtypes,
+            columns_to_serialize=columns_to_serialize,
+            session_timezone=self.session_timezone,
+        )
 
         # Append our values to the actual database table
         self.log(
@@ -654,7 +662,14 @@ class _TonyDBCOnlineOnly:
         with self.cursor() as cursor:
             cursor.executemany(update_command, update_command_values)
 
-    def update_blob(self, table_name, blob_column, id_value, filepath, max_size_MB=16):
+    def update_blob(
+        self,
+        table_name: str,
+        blob_column: str,
+        id_value,
+        filepath: str,
+        max_size_MB: int = 16,
+    ):
         """Add a BLOB (Binary Large OBject) to the database, from a file
         This requires that the table row already exists; we are just updating it
 
@@ -725,34 +740,9 @@ class _TonyDBCOnlineOnly:
 
         return df
 
-    @overload
-    def get_data(
-        self,
-        query: str,
-        before_retry_cmd: str | None = ...,
-        no_tracking: bool = ...,
-        return_type_codes: Literal[False] = ...,
-    ) -> list[dict[str, Any]]: ...
-
-    @overload
-    def get_data(
-        self,
-        query: str,
-        before_retry_cmd: str | None = ...,
-        no_tracking: bool = ...,
-        return_type_codes: Literal[True] = ...,
-    ) -> tuple[list[dict[str, Any]], dict[Any, str]]: ...
-
-    def get_data(
-        self,
-        query: str,
-        before_retry_cmd: str | None = None,
-        no_tracking: bool = False,
-        return_type_codes: bool = False,
-    ) -> Any:
-        if self.do_audit and not no_tracking:
-            started_at = self.now()
-
+    def _get_data_raw(
+        self, query: str, before_retry_cmd: str | None = None
+    ) -> tuple[list[dict[str, Any]], dict[Any, str]]:
         attempts_remaining = MAX_RECONNECTION_ATTEMPTS
         while attempts_remaining > 0:
             with self.cursor() as cursor:
@@ -822,6 +812,46 @@ class _TonyDBCOnlineOnly:
                 for warning in warnings:
                     self.log(warning)
 
+        return records0, fields
+
+    def get_type_codes(self, query: str) -> dict[str, str]:
+        """
+        Get a list of dicts with proper field names
+        (i.e. records in the pandas sense)
+        (e.g. {'media_object_id': 'LONGLONG', 'full_path': 'VAR_STRING'}
+        """
+        _, fields = self._get_data_raw(query=query)
+        type_codes = {v[0]: FIELD_TYPE_DICT[v[1]] for v in fields}
+        return type_codes
+
+    def get_data(
+        self, query: str, before_retry_cmd: str | None = None, no_tracking: bool = False
+    ) -> Any:
+        """
+        Get a list of dicts with proper field names, from a given database query.
+
+        Parameters:
+            query: the SQL query to execute (e.g. "SELECT id, pet FROM pets;")
+            before_retry_cmd: a command to execute before retrying the query
+            no_tracking: whether to track the query for audit purposes
+        Returns:
+            A list of dicts with the column names
+            [
+                {
+                    'id': 3,
+                    'pet': 'cat',
+                },
+            ]
+        """
+        if self.do_audit and not no_tracking:
+            started_at = self.now()
+
+        records0, fields = self._get_data_raw(
+            query=query, before_retry_cmd=before_retry_cmd
+        )
+
+        # Narrow our records to just the fields that are defined in cursor.description
+        # Frankly, this is a bit of a cargo-cult move; I don't remember why we do this
         records = [
             {fields[i][0]: field_value for i, field_value in enumerate(v)}
             for v in records0
@@ -836,22 +866,14 @@ class _TonyDBCOnlineOnly:
                 **get_payload_info(records),
             )
 
-        if return_type_codes:
-            # Get a list of dicts with proper field names
-            # (i.e. records in the pandas sense)
-            # (e.g. {'media_object_id': 'LONGLONG', 'full_path': 'VAR_STRING'}
-            type_codes = {v[0]: FIELD_TYPE_DICT[v[1]] for v in fields}
-
-            return records, type_codes
-        else:
-            return records
+        return records
 
     @property
-    def databases(self):
+    def databases(self) -> list[str]:
         return [d["Database"] for d in self.get_data("SHOW DATABASES;")]
 
     @property
-    def users(self):
+    def users(self) -> list[tuple[str, str]]:
         return [
             (d["Host"], d["User"])
             for d in self.get_data("SELECT * FROM mysql.user", no_tracking=True)
@@ -863,7 +885,7 @@ class _TonyDBCOnlineOnly:
         )
 
     @property
-    def production_databases(self):
+    def production_databases(self) -> list[str]:
         try:
             return self.__production_databases
         except AttributeError:
@@ -885,7 +907,7 @@ class _TonyDBCOnlineOnly:
 
             return self.__production_databases
 
-    def drop_database(self, database):
+    def drop_database(self, database: str) -> None:
         if database in self.production_databases:
             raise AssertionError(
                 f"DANGER DANGER!  You are trying to drop {database}, "
@@ -1140,10 +1162,24 @@ class _TonyDBCOnlineOnly:
             if len(result) == 0:
                 raise KeyError(f"Table {table} does not seem to exist.")
             # We need to map Types like "bigint(20)" to "bigint" to match our DATATYPE_MAP
-            col_datatypes = {
-                v["Field"]: DATATYPE_MAP[v["Type"].split("(")[0].strip().lower()]
-                for v in result
-            }
+            col_datatypes = {}
+            for v in result:
+                field_name = v["Field"]
+                sql_type = v["Type"].strip().lower()
+
+                # First try exact match (handles tinyint(1) and other specific cases)
+                if sql_type in DATATYPE_MAP:
+                    col_datatypes[field_name] = DATATYPE_MAP[sql_type]
+                else:
+                    # Try base type without size specification
+                    base_type = sql_type.split("(")[0].strip()
+                    if base_type in DATATYPE_MAP:
+                        col_datatypes[field_name] = DATATYPE_MAP[base_type]
+                    else:
+                        # Raise error for unknown types
+                        raise ValueError(
+                            f"Unknown SQL type '{sql_type}' for field '{field_name}'. Please add it to DATATYPE_MAP."
+                        )
             return col_datatypes
 
     @property
@@ -1180,14 +1216,14 @@ class _TonyDBCOnlineOnly:
             }
             return self._column_datatypes
 
-    def non_primary_keys(self, table):
+    def non_primary_keys(self, table: str):
         """Returns a list of all non-primary keys for a table"""
         pk = self.get_primary_key(table=table)
         cols = self.get_column_datatypes(table=table).keys()
         return [c for c in cols if c != pk]
 
     @property
-    def connection_params(self):
+    def connection_params(self) -> dict[str, Any]:
         """Useful for making a new connection.  A dict of all you need.
 
         e.g.
@@ -1204,7 +1240,7 @@ class _TonyDBCOnlineOnly:
             "media_to_deserialize": self.media_to_deserialize,
         }
 
-    def insert_row_all_string(self, table, row_dict):
+    def insert_row_all_string(self, table: str, row_dict: dict[str, str]) -> None:
         """Insert a single row into a table, as all string
         Note that the row_dict is required to be all string or stringifyable
         Note that it does not autocommit
@@ -1212,7 +1248,7 @@ class _TonyDBCOnlineOnly:
         if len(row_dict) == 0:
             return
 
-        stringified_row_dict = {
+        stringified_row_dict: dict[str, str] = {
             k: str(v)
             for k, v in row_dict.items()
             # Skip any columns which are to be NULL and rely on DEFAULT VALUE because it won't accept
@@ -1221,16 +1257,16 @@ class _TonyDBCOnlineOnly:
         }
         # We must sanitize the values to avoid things like single quotes breaking the INSERT
         # so we will pass a list of values to self.execute instead
-        command = f"""
+        command: str = f"""
             INSERT INTO {table}
                        ({", ".join(stringified_row_dict.keys())})
                 VALUES ({", ".join(["%s" for _ in stringified_row_dict])})
             """
-        command_values = list(map(str, stringified_row_dict.values()))
+        command_values: tuple[Any, ...] = tuple(map(str, stringified_row_dict.values()))
         self.execute(command=command, command_values=command_values)
 
     @contextmanager
-    def temp_id_table(self, id_list):
+    def temp_id_table(self, id_list: list[int]):
         """Create a temporary table filled with ids for JOINing purposes"""
         # Generate a random temporary table name
         temp_table_name = f"temp_loc_ids_{random.randint(1000, 9999)}"
@@ -1278,7 +1314,12 @@ class _TonyDBCOnlineOnly:
         else:
             columns_to_serialize = []
 
-        df_serialized = serialize_table(df, col_dtypes, columns_to_serialize)
+        df_serialized = serialize_table(
+            cur_df=df,
+            col_dtypes=col_dtypes,
+            columns_to_serialize=columns_to_serialize,
+            session_timezone=self.session_timezone,
+        )
         payload_info = get_payload_info(df_serialized)
         payload_size_MB = payload_info["payload_size"] / (1024**2)
 
@@ -1685,9 +1726,8 @@ class TonyDBC(_TonyDBCOnlineOnly):
     def get_data(
         self,
         query: str,
-        before_retry_cmd=None,
-        no_tracking=False,
-        return_type_codes=False,
+        before_retry_cmd: str | None = None,
+        no_tracking: bool = False,
     ) -> Any:
         if not self.is_online:
             raise AssertionError("get_data can only be used while online")
@@ -1696,10 +1736,9 @@ class TonyDBC(_TonyDBCOnlineOnly):
                 query=query,
                 before_retry_cmd=before_retry_cmd,
                 no_tracking=no_tracking,
-                return_type_codes=return_type_codes,
             )
 
-    def drop_database(self, database) -> None:
+    def drop_database(self, database: str) -> None:
         if not self.is_online:
             raise AssertionError("Cannot drop database if not online")
         else:
@@ -1776,8 +1815,8 @@ class TonyDBC(_TonyDBCOnlineOnly):
         command: str,
         command_values=None,
         before_retry_cmd=None,
-        no_tracking=False,
-        log_progress=False,
+        no_tracking: bool = False,
+        log_progress: bool = False,
     ) -> None:
         kwargs = {
             "command": command,
@@ -1804,19 +1843,19 @@ class TonyDBC(_TonyDBCOnlineOnly):
         else:
             self.__update_queue.put(("execute_script", kwargs))
 
-    def insert_row_all_string(self, table, row_dict) -> None:
+    def insert_row_all_string(self, table: str, row_dict: dict[str, str]) -> None:
         kwargs = {
             "table": table,
             "row_dict": row_dict,
         }
 
         if self.is_online:
-            super(TonyDBC, self).insert_row_all_string(**kwargs)
+            super(TonyDBC, self).insert_row_all_string(table, row_dict)
         else:
             self.__update_queue.put(("insert_row_all_string", kwargs))
 
     def append_to_table(
-        self, table, df, return_reindexed=False, no_tracking=False
+        self, table: str, df: pd.DataFrame, return_reindexed=False, no_tracking=False
     ) -> pd.DataFrame | None:
         kwargs = {
             "table": table,
